@@ -7,7 +7,7 @@ RDF::Query::Algebra - Base class for Algebra expressions
 
 =head1 VERSION
 
-This document describes RDF::Query::Algebra version 2.906.
+This document describes RDF::Query::Algebra version 2.908.
 
 =head1 METHODS
 
@@ -19,7 +19,7 @@ package RDF::Query::Algebra;
 
 our (@ISA, @EXPORT_OK);
 BEGIN {
-	our $VERSION	= '2.906';
+	our $VERSION	= '2.908';
 	
 	require Exporter;
 	@ISA		= qw(Exporter);
@@ -67,6 +67,8 @@ use RDF::Query::Algebra::Update;
 use RDF::Query::Algebra::Minus;
 use RDF::Query::Algebra::Sequence;
 use RDF::Query::Algebra::Create;
+use RDF::Query::Algebra::Copy;
+use RDF::Query::Algebra::Move;
 
 use constant SSE_TAGS	=> {
 	'BGP'					=> 'RDF::Query::Algebra::BasicGraphPattern',
