@@ -7,7 +7,7 @@ RDF::Query::Algebra::Path - Algebra class for path patterns
 
 =head1 VERSION
 
-This document describes RDF::Query::Algebra::Path version 2.907.
+This document describes RDF::Query::Algebra::Path version 2.908.
 
 =cut
 
@@ -27,7 +27,7 @@ use Carp qw(carp croak confess);
 our ($VERSION, $debug, $lang, $languri);
 BEGIN {
 	$debug		= 0;
-	$VERSION	= '2.907';
+	$VERSION	= '2.908';
 }
 
 ######################################################################
@@ -236,7 +236,7 @@ sub _expand_path {
 		} elsif ($op =~ /^(\d+)-$/) {
 			$psse	= join('/', @nodessse) . "{$1,}";
 		} else {
-			die "Serialization of unknown path type $op";
+			confess "Serialization of unknown path type $op";
 		}
 		return $psse;
 	}
