@@ -7,7 +7,7 @@ RDF::Query::Algebra::Service - Algebra class for SERVICE (federation) patterns
 
 =head1 VERSION
 
-This document describes RDF::Query::Algebra::Service version 2.907.
+This document describes RDF::Query::Algebra::Service version 2.908.
 
 =cut
 
@@ -32,7 +32,7 @@ use RDF::Trine::Iterator qw(sgrep smap swatch);
 our ($VERSION, $BLOOM_FILTER_ERROR_RATE);
 BEGIN {
 	$BLOOM_FILTER_ERROR_RATE	= 0.1;
-	$VERSION	= '2.907';
+	$VERSION	= '2.908';
 }
 
 ######################################################################
@@ -57,7 +57,8 @@ sub new {
 	my $endpoint	= shift;
 	my $pattern		= shift;
 	my $silent		= shift || 0;
-	return bless( [ 'SERVICE', $endpoint, $pattern, $silent ], $class );
+	my $ggp			= shift;
+	return bless( [ 'SERVICE', $endpoint, $pattern, $silent, $ggp ], $class );
 }
 
 =item C<< construct_args >>
@@ -69,7 +70,7 @@ will produce a clone of this algebra pattern.
 
 sub construct_args {
 	my $self	= shift;
-	return ($self->endpoint, $self->pattern);
+	return ($self->endpoint, $self->pattern, $self->silent, $self->lhs);
 }
 
 =item C<< endpoint >>
@@ -114,6 +115,19 @@ sub silent {
 	return $self->[3];
 }
 
+=item C<< lhs >>
+
+If the SERVCE operation uses a variable endpoint, then it is considered a binary
+operator, executing the left-hand-side pattern first, and using results from it
+to bind endpoint URL values to use in SERVICE evaluation.
+
+=cut
+
+sub lhs {
+	my $self	= shift;
+	return $self->[4];
+}
+
 =item C<< add_bloom ( $variable, $filter ) >>
 
 Adds a FILTER to the enclosed GroupGraphPattern to restrict values of the named
@@ -155,11 +169,20 @@ sub sse {
 	my $prefix	= shift || '';
 	my $indent	= $context->{indent};
 	
-	return sprintf(
-		"(service\n${prefix}${indent}%s\n${prefix}${indent}%s)",
-		$self->endpoint->sse( $context, "${prefix}${indent}" ),
-		$self->pattern->sse( $context, "${prefix}${indent}" )
-	);
+	if (my $ggp = $self->lhs) {
+		return sprintf(
+			"(service\n${prefix}${indent}%s\n${prefix}${indent}%s\n${prefix}${indent}%s)",
+			$self->lhs->sse( $context, "${prefix}${indent}" ),
+			$self->endpoint->sse( $context, "${prefix}${indent}" ),
+			$self->pattern->sse( $context, "${prefix}${indent}" )
+		);
+	} else {
+		return sprintf(
+			"(service\n${prefix}${indent}%s\n${prefix}${indent}%s)",
+			$self->endpoint->sse( $context, "${prefix}${indent}" ),
+			$self->pattern->sse( $context, "${prefix}${indent}" )
+		);
+	}
 }
 
 =item C<< as_sparql >>
@@ -173,13 +196,25 @@ sub as_sparql {
 	my $context	= shift;
 	my $indent	= shift;
 	my $op		= ($self->silent) ? 'SERVICE SILENT' : 'SERVICE';
-	my $string	= sprintf(
-		"%s %s %s",
-		$op,
-		$self->endpoint->as_sparql( $context, $indent ),
-		$self->pattern->as_sparql( { %$context, force_ggp_braces => 1 }, $indent ),
-	);
-	return $string;
+	if (my $ggp = $self->lhs) {
+		local($context->{skip_filter})	= 0;
+		my $string	= sprintf(
+			"%s\n${indent}%s %s %s",
+			$ggp->as_sparql( $context, $indent ),
+			$op,
+			$self->endpoint->as_sparql( $context, $indent ),
+			$self->pattern->as_sparql( { %$context, force_ggp_braces => 1 }, $indent ),
+		);
+		return $string;
+	} else {
+		my $string	= sprintf(
+			"%s %s %s",
+			$op,
+			$self->endpoint->as_sparql( $context, $indent ),
+			$self->pattern->as_sparql( { %$context, force_ggp_braces => 1 }, $indent ),
+		);
+		return $string;
+	}
 }
 
 =item C<< as_hash >>
@@ -195,6 +230,7 @@ sub as_hash {
 		type 		=> lc($self->type),
 		endpoint	=> $self->endpoint,
 		pattern		=> $self->pattern->as_hash,
+		lhs			=> $self->lhs->as_hash,
 	};
 }
 
@@ -216,7 +252,7 @@ Returns a list of the variable names used in this algebra expression.
 
 sub referenced_variables {
 	my $self	= shift;
-	my @list	= $self->pattern->referenced_variables;
+	my @list	= RDF::Query::_uniq($self->pattern->referenced_variables, $self->lhs->referenced_variables);
 	return @list;
 }
 
@@ -229,7 +265,11 @@ bind values during execution.
 
 sub potentially_bound {
 	my $self	= shift;
-	return $self->pattern->potentially_bound;
+	my @list	= RDF::Query::_uniq($self->pattern->potentially_bound);
+	if ($self->lhs) {
+		push(@list, $self->lhs->potentially_bound);
+	}
+	return @list;
 }
 
 =item C<< definite_variables >>
@@ -243,6 +283,7 @@ sub definite_variables {
 	return RDF::Query::_uniq(
 		map { $_->name } grep { $_->isa('RDF::Query::Node::Variable') } ($self->graph),
 		$self->pattern->definite_variables,
+		($self->lhs ? $self->lhs->definite_variables : ()),
 	);
 }
 
@@ -260,10 +301,15 @@ sub qualify_uris {
 	my $ns		= shift;
 	my $base_uri	= shift;
 	
-	my $pattern	= $self->pattern->qualify_uris( $ns, $base_uri );
+	my $pattern		= $self->pattern->qualify_uris( $ns, $base_uri );
 	my $endpoint	= $self->endpoint;
+	my $silent		= $self->silent;
 	my $uri	= $endpoint->uri;
-	return $class->new( $endpoint, $pattern );
+	if (my $ggp = $self->lhs) {
+		return $class->new( $endpoint, $pattern, $silent, $ggp->qualify_uris($ns, $base_uri) );
+	} else {
+		return $class->new( $endpoint, $pattern, $silent );
+	}
 }
 
 1;
