@@ -7,7 +7,7 @@ RDF::Query::ExecutionContext - Query execution context
 
 =head1 VERSION
 
-This document describes RDF::Query::ExecutionContext version 2.906.
+This document describes RDF::Query::ExecutionContext version 2.908.
 
 =head1 METHODS
 
@@ -24,7 +24,7 @@ use warnings;
 
 our ($VERSION);
 BEGIN {
-	$VERSION	= '2.906';
+	$VERSION	= '2.908';
 }
 
 ######################################################################
@@ -198,6 +198,15 @@ sub optimistic_threshold_time {
 	return $self->_get_value( 'optimistic_threshold_time', @_ );
 }
 
+=item C<< delegate >>
+
+=cut
+
+sub delegate {
+	my $self	= shift;
+	return $self->_get_value( 'delegate', @_ );
+}
+
 sub _get_value {
 	my $self	= shift;
 	my $key		= shift;
