@@ -7,7 +7,7 @@ RDF::Query::Node::Variable - RDF Node class for variables
 
 =head1 VERSION
 
-This document describes RDF::Query::Node::Variable version 2.906.
+This document describes RDF::Query::Node::Variable version 2.908.
 
 =cut
 
@@ -26,7 +26,7 @@ use Carp qw(carp croak confess);
 
 our ($VERSION);
 BEGIN {
-	$VERSION	= '2.906';
+	$VERSION	= '2.908';
 }
 
 ######################################################################
@@ -130,6 +130,10 @@ sub referenced_variables {
 	return RDF::Query::_uniq(@vars);
 }
 
+sub nonaggregated_referenced_variables {
+	return;
+}
+
 sub DESTROY {
 	my $self	= shift;
 	delete $vars{ refaddr($self) };
