@@ -7,7 +7,7 @@ RDF::Query::Plan - Executable query plan nodes.
 
 =head1 VERSION
 
-This document describes RDF::Query::Plan version 2.906.
+This document describes RDF::Query::Plan version 2.908.
 
 =head1 METHODS
 
@@ -21,7 +21,7 @@ use strict;
 use warnings;
 use Data::Dumper;
 use List::Util qw(reduce);
-use Scalar::Util qw(blessed reftype);
+use Scalar::Util qw(blessed reftype refaddr);
 use RDF::Query::Error qw(:try);
 use RDF::Query::BGPOptimizer;
 
@@ -52,6 +52,8 @@ use RDF::Query::Plan::Minus;
 use RDF::Query::Plan::Sequence;
 use RDF::Query::Plan::Path;
 use RDF::Query::Plan::NamedGraph;
+use RDF::Query::Plan::Copy;
+use RDF::Query::Plan::Move;
 
 use RDF::Trine::Statement;
 use RDF::Trine::Statement::Quad;
@@ -62,9 +64,12 @@ use constant CLOSED		=> 0x04;
 
 ######################################################################
 
-our ($VERSION);
+our ($VERSION, %PLAN_CLASSES);
 BEGIN {
-	$VERSION	= '2.906';
+	$VERSION		= '2.908';
+	%PLAN_CLASSES	= (
+		service	=> 'RDF::Query::Plan::Service',
+	);
 }
 
 ######################################################################
@@ -158,18 +163,19 @@ sub explain {
 	}
 	my $indent	= '' . ($s x $count);
 	my $type	= $self->plan_node_name;
-	my $string	= "${indent}${type}\n";
+	my $string	= sprintf("%s%s (0x%x)\n", $indent, $type, refaddr($self));
 	foreach my $p ($self->plan_node_data) {
 		if (blessed($p)) {
 			if ($p->isa('RDF::Trine::Statement::Quad')) {
-				$string	.= "${indent}${s}" . $p->as_string . "\n";
+				$string	.= "${indent}${s}" . join(' ', map { ($_->isa('RDF::Trine::Node::Nil')) ? "(nil)" : $_->as_sparql } $p->nodes) . "\n";
 			} elsif ($p->isa('RDF::Trine::Node::Nil')) {
 				$string	.= "${indent}${s}(nil)\n";
 			} else {
 				$string	.= $p->explain( $s, $count+1 );
 			}
 		} elsif (ref($p)) {
-			warn 'unexpected non-blessed ref in RDF::Query::Plan->explain: ' . Dumper($p);
+			$string	.= "${indent}${s}" . Dumper($p);
+			Carp::cluck 'unexpected non-blessed ref in RDF::Query::Plan->explain: ' . Dumper($p);
 		} else {
 			no warnings 'uninitialized';
 			$string	.= "${indent}${s}$p\n";
@@ -307,6 +313,17 @@ sub serialize {
 	
 }
 
+=item C<< delegate >>
+
+Returns the delegate object if available.
+
+=cut
+
+sub delegate {
+	my $self	= shift;
+	return $self->[0]{delegate};
+}
+
 =item C<< referenced_variables >>
 
 Returns a list of variable names that are referenced by this plan.
@@ -455,7 +472,7 @@ sub generate_plans {
 	############################################################################
 	
 	my ($project);
-	my $constant	= $args{ constants };
+	my $constant	= delete $args{ constants };
 	
 	if ($algebra->isa('RDF::Query::Algebra::Sort') or not($algebra->is_solution_modifier)) {
 		# projection has to happen *after* sorting, since a sort expr might reference a variable that we project away
@@ -595,7 +612,8 @@ sub generate_plans {
 						my $plan	= $join_type->new( $a, $b, 1, {} );
 						push( @plans, $plan );
 					} catch RDF::Query::Error::MethodInvocationError with {
-#						warn "caught MethodInvocationError.";
+# 						my $e	= shift;
+# 						warn "caught MethodInvocationError: " . Dumper($e);
 					};
 				}
 			}
@@ -648,16 +666,37 @@ sub generate_plans {
 		my @base	= $self->generate_plans( $pattern, $context, %args );
 		my @plans;
 		foreach my $plan (@base) {
-			my $ns			= $context->ns;
-			my $pstr		= $pattern->as_sparql({namespaces => $ns}, '');
-			unless (substr($pstr, 0, 1) eq '{') {
-				$pstr	= "{ $pstr }";
+			my $sparqlcb	= sub {
+				my $row		= shift;
+				my $p		= $pattern;
+				if ($row) {
+					$p		= $p->bind_variables( $row );
+				}
+				my $ns			= $context->ns;
+				my $pstr		= $p->as_sparql({namespaces => $ns}, '');
+				unless (substr($pstr, 0, 1) eq '{') {
+					$pstr	= "{ $pstr }";
+				}
+				my $sparql		= join("\n",
+									(map { sprintf("PREFIX %s: <%s>", ($_ eq '__DEFAULT__' ? '' : $_), $ns->{$_}) } (keys %$ns)),
+									sprintf("SELECT * WHERE %s", $pstr)
+								);
+				return $sparql;
+			};
+			
+# 			unless ($algebra->endpoint->can('uri_value')) {
+# 				throw RDF::Query::Error::UnimplementedError (-text => "Support for variable-endpoint SERVICE blocks is not implemented");
+# 			}
+			
+			if (my $ggp = $algebra->lhs) {
+				my @lhs_base	= $self->generate_plans( $ggp, $context, %args );
+				foreach my $lhs_plan (@lhs_base) {
+					my $splan	= RDF::Query::Plan::Service->new( $algebra->endpoint, $plan, $algebra->silent, $sparqlcb, $lhs_plan );
+					push(@plans, $splan);
+				}
+			} else {
+				push(@plans, $PLAN_CLASSES{'service'}->new( $algebra->endpoint, $plan, $algebra->silent, $sparqlcb ));
 			}
-			my $sparql		= join("\n",
-								(map { sprintf("PREFIX %s: <%s>", $_, $ns->{$_}) } (keys %$ns)),
-								sprintf("SELECT * WHERE %s", $pstr)
-							);
-			push(@plans, RDF::Query::Plan::Service->new( $algebra->endpoint->uri_value, $plan, $sparql ));
 		}
 		push(@return_plans, @plans);
 	} elsif ($type eq 'SubSelect') {
@@ -766,6 +805,12 @@ sub generate_plans {
 	} elsif ($type eq 'Create') {
 		my $plan	= RDF::Query::Plan::Constant->new();
 		push(@return_plans, $plan);
+ 	} elsif ($type eq 'Copy') {
+ 		my $plan	= RDF::Query::Plan::Copy->new( $algebra->from, $algebra->to, $algebra->silent );
+		push(@return_plans, $plan);
+ 	} elsif ($type eq 'Move') {
+ 		my $plan	= RDF::Query::Plan::Move->new( $algebra->from, $algebra->to, $algebra->silent );
+		push(@return_plans, $plan);
 	} else {
 		throw RDF::Query::Error::MethodInvocationError (-text => "Cannot generate an execution plan for unknown algebra class $aclass");
 	}
@@ -837,6 +882,7 @@ sub _join_plans {
 						}
 					}
 					foreach my $join_type (@join_types) {
+						next if ($join_type eq 'RDF::Query::Plan::Join::PushDownNestedLoop' and $b->subplans_of_type('RDF::Query::Plan::Service'));
 						try {
 							my @algebras;
 							foreach ($algebra_a, $algebra_b) {
@@ -1182,6 +1228,38 @@ the signature returned by C<< plan_prototype >>.
 
 sub plan_node_data;
 
+
+=item C<< subplans_of_type ( $type [, $block] ) >>
+
+Returns a list of Plan objects matching C<< $type >> (tested with C<< isa >>).
+If C<< $block >> is given, then matching stops descending a subtree if the current
+node is of type C<< $block >>, continuing matching on other subtrees.
+This list includes the current plan object if it matches C<< $type >>, and is
+generated in infix order.
+
+=cut
+
+sub subplans_of_type {
+	my $self	= shift;
+	my $type	= shift;
+	my $block	= shift;
+	
+	return if ($block and $self->isa($block));
+	
+	my @patterns;
+	push(@patterns, $self) if ($self->isa($type));
+	
+	
+	foreach my $p ($self->plan_node_data) {
+		if (blessed($p) and $p->isa('RDF::Query::Plan')) {
+			push(@patterns, $p->subplans_of_type($type, $block));
+		}
+	}
+	return @patterns;
+}
+
+
+
 1;
 
 __END__
